source("modules/Loading/Loading.R")
source("modules/Skill/Skill.R")
source("modules/Downscaling/Downscaling.R")
source("modules/Saving/Saving.R")
source("modules/Anomalies/Anomalies.R")
source("SUNSET_PROV/R/SunsetProv.Command.R")

library(igraph)
library(rjson)
library(testthat)

recipe_file <- "tests_provenance/recipes/recipe_test_provenance3.yml"

suppressWarnings({invisible(capture.output(
  recipe <- prepare_outputs(recipe_file, disable_checks = F)
))})

#Load datasets
suppressWarnings({invisible(capture.output(
  data <- Loading(recipe)
))})

#Compute anomalies
suppressWarnings({invisible(capture.output(
  data_ano <- Anomalies(recipe, data)
))})

#Downscale the data
suppressWarnings({invisible(capture.output(
  downscaled_data <- Downscaling(recipe, data_ano)
))})

#Compute skill metrics
suppressWarnings({invisible(capture.output(
  skill_metrics <- Skill(recipe, downscaled_data)
))})

#Generate JSON file
graph2json(skill_metrics$graph.prov$graph,
           "/esarchive/scratch/apuiggro/sunset/tests_provenance/out-logs/json_test3.json")


#Make JSON readable
json_file3 <- fromJSON(file = "tests_provenance/out-logs/json_test3.json")

#======================================

test_that("1. Loading", {

  expect_equal(
    is.list(data$graph.prov),
    TRUE
  )
  expect_equal(
    class(data$graph.prov$graph),
    "igraph"
  )
  expect_equal(
    data$graph.prov$parentnodename$parentnodename_hcst,
    "DatasetSubset_hcst"
  )
  expect_equal(
    data$graph.prov$parentnodename$parentnodename_obs,
    "DatasetSubset_obs"
  )
  expect_equal(
    length(V(data$graph.prov$graph)$name),
    50
  )
  expect_equal(
    V(data$graph.prov$graph)$name[[9]],
    "VerticalExtenthcst"
    
  )
  expect_equal(
    V(data$graph.prov$graph)$name[[5]],
    "Location_Dataset_hcst"
  )
})


#======================================

test_that("2. Anomalies", {

  expect_equal(
    is.list(data_ano$graph.prov),
    TRUE
  )
  expect_equal(
    class(data_ano$graph.prov$graph),
    "igraph"
  )
  expect_equal(
    data_ano$graph.prov$parentnodename$parentnodename_hcst,
    "Anomalies_hcst"
  )
  expect_equal(
    data_ano$graph.prov$parentnodename$parentnodename_obs,
    "Anomalies_obs"
  )
  expect_equal(
    length(V(data_ano$graph.prov$graph)$name),
    60
  )
  expect_equal(
    V(data_ano$graph.prov$graph)$name[[54]],
    "Anomalies_obs"
  )
  expect_equal(
    V(data_ano$graph.prov$graph)$name[[53]],
    "Climatology_obs"
  )
})

#======================================


test_that("3. Downscaling", {

  expect_equal(
    is.list(downscaled_data$graph.prov),
    TRUE
  )
  expect_equal(
    class(downscaled_data$graph.prov$graph),
    "igraph"
  )
  expect_equal(
    downscaled_data$graph.prov$parentnodename$parentnodename_hcst,
    "Downscaling_hcst"
  )
  expect_equal(
    downscaled_data$graph.prov$parentnodename$parentnodename_obs,
    "Anomalies_obs"
  )
  expect_equal(
    length(V(downscaled_data$graph.prov$graph)$name),
    68
  )
  expect_equal(
    V(downscaled_data$graph.prov$graph)$name[[62]],
    "SpatialExtent_Downscaling_hcst"
  )
})

#======================================

test_that("4. Skill", {

  expect_equal(
    is.list(skill_metrics$graph.prov),
    TRUE
  )
  expect_equal(
    class(skill_metrics$graph.prov$graph),
    "igraph"
  )
  expect_equal(
    skill_metrics$graph.prov$parentnodename$parentnodename_hcst,
    "Verification_hcst"
  )
  expect_equal(
    skill_metrics$graph.prov$parentnodename$parentnodename_obs,
    "Anomalies_obs"
  )
  expect_equal(
    length(V(skill_metrics$graph.prov$graph)$name),
    79
  )
  expect_equal(
    V(skill_metrics$graph.prov$graph)$name[[72]],
    "Metrics_hcst_rpss"
  )
  expect_equal(
    V(skill_metrics$graph.prov$graph)$name[[73]],
    "Metrics_hcst_mean_bias"
  )

})

#======================================

test_that("5. JSON", {

  expect_equal(
    json_file3[[1]]$ds,
    "http://www.metaclip.org/datasource/datasource.owl#"
  )
  expect_equal(
    json_file3[[2]][[1]][[2]],
    "ds:SeasonalHindcast"
  )
  expect_equal(
    json_file3[[2]][[3]][[3]],
    "Brier Skill Score Lower Extreme"
  )
  expect_equal(
    json_file3[[2]][[4]][[3]],
    "Continuous Ranked Probability Skill Score"
  )
  expect_equal(
    json_file3[[2]][[6]][[1]],
    "#Metrics_hcst_mean_bias"
  )
  expect_equal(
    json_file3[[2]][[5]][[5]][[1]],
    "#Verification_hcst"
  )

})

print("Test 3 successfull")